@@ -14,8 +14,9 @@
 #    under the License.
 
 from neutron.common import rpc as n_rpc
-from neutron.common import utils as n_utils
+from neutron import service
 from neutron_lib.api.definitions import firewall as fw_ext
+from neutron_lib.api import extensions
 from neutron_lib import constants as nl_constants
 from neutron_lib import context as neutron_context
 from neutron_lib.exceptions import firewall_v1 as f_exc
@@ -155,7 +156,6 @@ class FirewallPlugin(
 
     def __init__(self):
         """Do the initialization for the firewall service plugin here."""
-        self.start_rpc_listeners()
 
         self.agent_rpc = FirewallAgentApi(
             f_const.FW_AGENT,
@@ -163,47 +163,28 @@ class FirewallPlugin(
         )
         firewall_db.subscribe()
 
+        rpc_worker = service.RpcWorker([self], worker_process_count=0)
+        self.add_worker(rpc_worker)
+
     def start_rpc_listeners(self):
         self.endpoints = [FirewallCallbacks(self)]
 
-        self.conn = n_rpc.create_connection()
+        self.conn = n_rpc.Connection()
         self.conn.create_consumer(
             f_const.FIREWALL_PLUGIN, self.endpoints, fanout=False)
         return self.conn.consume_in_threads()
 
-    def _check_dvr_extensions(self, l3plugin):
-        return (
-            n_utils.is_extension_supported(
-                l3plugin, nl_constants.L3_AGENT_SCHEDULER_EXT_ALIAS) and
-            n_utils.is_extension_supported(
-                l3plugin, nl_constants.L3_DISTRIBUTED_EXT_ALIAS) and
-            getattr(l3plugin, '_get_dvr_hosts_for_router', False))
-
     def _get_hosts_to_notify(self, context, router_ids):
         """Returns all hosts to send notification about firewall update"""
         l3_plugin = directory.get_plugin(plugin_constants.L3)
         no_broadcast = (
-            n_utils.is_extension_supported(
+            extensions.is_extension_supported(
                 l3_plugin, nl_constants.L3_AGENT_SCHEDULER_EXT_ALIAS) and
             getattr(l3_plugin, 'get_l3_agents_hosting_routers', False))
-        scheduled_hosts = set()
         if no_broadcast:
-            # This call checks for all scheduled routers to the network node
             agents = l3_plugin.get_l3_agents_hosting_routers(
                 context, router_ids, admin_state_up=True, active=True)
-            scheduled_hosts = set([a.host for a in agents])
-
-        # Now check for unscheduled DVR router on distributed compute hosts
-        unscheduled_dvr_hosts = set()
-        dvr_broadcast = self._check_dvr_extensions(l3_plugin)
-        if (dvr_broadcast):
-            for router_id in router_ids:
-                hosts = set(l3_plugin._get_dvr_hosts_for_router(
-                    context, router_id))
-                unscheduled_dvr_hosts |= hosts
-        if no_broadcast or dvr_broadcast:
-            scheduled_hosts = scheduled_hosts.union(unscheduled_dvr_hosts)
-            return scheduled_hosts
+            return [a.host for a in agents]
 
         # NOTE(blallau): default: FirewallAgentAPI performs RPC broadcast
         return [None]
